# SessionCleanup
# Cleans up stale sessions and related files from within a Rails application
# Version 0.3.0
# Copyright 2006 by Al Evans <anejr@alevans.com>
# Released under the MIT license


class SessionCleanup
  
  # How many hours to keep sessions around
  HOURS_TO_KEEP = 24
  # SessionCleanup only runs once every RUN_INTERVAL hours
  RUN_INTERVAL = 8
  # The ctime of this file is changed on each run
  LAST_CLEANUP_FILE = "#{RAILS_ROOT}/log/last_session_cleanup"
  
  # Call SessionCleanup.cleanup at any convenient point in your application
  # where an extra second or two won't matter. It will delete sessions that
  # are over HOURS_TO_KEEP hours old, once every RUN_INTERVAL hours
  #
  # If there is extra data -- for example, a work file -- associated with a
  # session, you can pass a block with one parameter like this:
  #   SessionCleanup.cleanup {|h| File.delete(h[:workfile])}
  # The block will be called with the session hash for each session being
  # deleted.
  #
  # WARNING!! SessionCleanup does not know whether a session is current or
  # even in use! Be sure to set HOURS_TO_KEEP to anumber of hours after which
  # you are CERTAIN the session is stale.
  
  def SessionCleanup.cleanup
    if (last_cleanup_time < RUN_INTERVAL.hours.ago)
      p = nil
      p = Proc.new if block_given?
      case ActionController::Base.session_store.to_s
      when 'CGI::Session::ActiveRecordStore'
        database_cleanup(p)
      when 'CGI::Session::PStore'
        pstore_cleanup(p)
      end
      # TODO: Should handle other session storage types
      File.open(LAST_CLEANUP_FILE, "w").close #touch file
    end
  end
  
  def SessionCleanup.last_cleanup_time
    if File.exist?(LAST_CLEANUP_FILE)
      return File.ctime(LAST_CLEANUP_FILE)
    else
      return Time.at(0)
    end
  end

  def SessionCleanup.database_cleanup(cleanup_proc)
    unless cleanup_proc.nil?
       CGI::Session::ActiveRecordStore::Session.find(:all, 
        :conditions => [ "updated_at < ?", HOURS_TO_KEEP.hours.ago]).each do |e|
        h = Marshal.load(Base64.decode64(e['data']))
        cleanup_proc.call(h)
      end
    end
    CGI::Session::ActiveRecordStore::Session.delete_all( ['updated_at < ?', HOURS_TO_KEEP.hours.ago ] )
  end

  def SessionCleanup.pstore_cleanup(cleanup_proc)
    session_dir = ActionController::Base.session_options[:tmpdir]
    Dir.new(session_dir).each do |e|
      if e =~ %r{#{ActionController::Base.session_options[:prefix]}} &&
        File.ctime("#{session_dir}/#{e}") < HOURS_TO_KEEP.hours.ago
        f_path = "#{session_dir}/#{e}"
        unless cleanup_proc.nil?
          p = ::PStore.new(f_path)
          p.transaction do
            h = p['hash']
            cleanup_proc.call(h)
          end
        end
        File.delete(f_path)
      end
    end
  end

  private_class_method :database_cleanup, :pstore_cleanup, :last_cleanup_time

end
